import axios from 'axios'

function randomString(len) {
  len = len || 10
  var chars = 'ABCDEFGHJKMNPQRSTWXYZabcdefhijkmnprstwxyz123456789'
  var maxPos = chars.length
  let pwd = ''
  for (let i = 0; i < len; i++) {
    pwd = pwd + chars.charAt(Math.floor(Math.random() * maxPos))
  }
  return pwd + new Date().getTime()
}

const velog = {
  reportTimes: 0,
  isJson: true,
  apiUrl: '',
  appName: '',
  appId: '',
  options: {
    markUser: '',
    isFirstIn: true,
    message: '',
    url: location.href,
    level: 'error',
    timestamp: '',
    user_agent: navigator.userAgent,
    stack: '', // error stack
    data: {} // other data
  },
  installed: false,
  markUser() {
    let markUser = localStorage.getItem('velog__markuser') || ''
    velog.options.markUser = markUser
    velog.options.isFirstIn = false
    if (!markUser) {
      markUser = randomString()
      localStorage.setItem('velog__markuser', markUser)
      velog.options.markUser = markUser
    }
  },
  processStackMsg(error) {
    let stack = error.stack
      .replace(/\n/gi, '') // 去掉换行
      .replace(/\bat\b/gi, '@') // chrome 中是 at, firefox 是 @
      .split('@') // 以@分割信息
      .slice(0, 9) // 最大堆栈长度 (Error.stackTraceLimit = 10), 只取前10条
      .map(v => v.replace(/^\s*|\s*$/g, '')) // 去掉多余空格
      .join('~') // 手动添加分隔符
      .replace(/\?[^:]+/gi, '') // 去掉js文件链接多余参数(?t=2 eg.)
    const msg = error.toString()
    if (stack.indexOf(msg) < 0) {
      stack = msg + '@' + stack
    }
    return stack
  },
  formatComponentName(vm) {
    if (vm.$root === vm) return 'root'
    var name = vm._isVue
      ? (vm.$options && vm.$options.name) ||
        (vm.$options && vm.$options._compeontTag)
      : vm.name
    return (
      (name ? 'component <' + name + '>' : 'anonymous component') +
      (vm._isVue && vm.$options && vm.$options.__file
        ? ' at ' + (vm.$options && vm.$options.__file)
        : '')
    )
  },
  init(Vue) {
    // init user
    velog.markUser()
    // 监控资源加载错误(img, script, css, jsonp)
    window.addEventListener(
      'error',
      event => {
        const target = event.target ? event.target : event.srcElement
        velog.options.message = event.target.localName + ' is load error'
        velog.options.stack = 'resource is not found'
        velog.options.data = {
          type: event.type,
          tagName: event.target.localName,
          html: target.outerHTML,
          source: event.target.src,
          fileName: event.target.currentSrc,
          category: 'resource'
        }
        if (event.target !== window) {
          const reportData = Object.assign({}, velog.options)
          velog.sendReport(reportData)
        }
      },
      true
    )
    // 监控js错误
    window.onerror = function(message, source, lineno, colno, error) {
      if (message === 'Script error.' && !source) {
        return false
      }
      // async
      setTimeout(function() {
        colno = colno || (window.event && window.event.errorCharacter) || 0
        if (error && error.stack) {
          velog.options.message = message
          velog.options.stack = error.stack
        } else {
          velog.options.message = message
          velog.options.stack = ''
        }
        velog.options.data = {
          source: source,
          category: 'javascript',
          line: lineno,
          col: colno
        }
        const reportData = Object.assign({}, velog.options)
        velog.sendReport(reportData)
      }, 0)
      return true // 错误不抛出到控制台
    }
    // promise errors
    window.addEventListener(
      'unhandledrejection',
      event => {
        velog.options.message = event.reason
        velog.options.data = {
          url: location.href,
          category: 'promise'
        }
        velog.options.stack = 'promise is error'
        const reportData = Object.assign({}, velog.options)
        velog.sendReport(reportData)
        event.preventDefault() // 阻止控制台抛错
      },
      true
    )
    // vue errors
    Vue.config.errorHandler = (error, vm, info) => {
      const componentName = velog.formatComponentName(vm)
      velog.options.message = error.message
      velog.options.stack = velog.processStackMsg(error)
      velog.options.data = {
        category: 'vue',
        componentName: componentName,
        info: info
      }
      const reportData = Object.assign({}, velog.options)
      velog.sendReport(reportData)
    }
  },
  send(error) {
    // send page catch error
    velog.options.message = error.message
    velog.options.data = {
      url: location.href,
      category: 'ajax'
    }
    velog.options.stack = 'promise is error'
    const reportData = Object.assign({}, velog.options)
    velog.sendReport(reportData)
  },
  sendReport(data) {
    const oData = Object.assign({}, data, {
      timestamp: new Date().getTime(),
      url: location.href
    })
    const postData = {
      title: velog.appName,
      content: oData,
      type: velog.appId
    }
    axios
      .post(velog.apiUrl, postData)
      .then(response => {
        if (response.data) {
          velog.reportTimes++
        }
      })
      .catch(err => {
        if (err) {
          console.log('rp error failed')
        }
      })
  }
}

function install(Vue, options) {
  if (velog.installed) return false
  let { isReport, isJson, apiUrl, appName, appId } = options
  if (!apiUrl) {
    return console.error(`api url is required`)
  }
  if (!appId) {
    return console.error(`app id is required`)
  }
  velog.isJson = isJson
  velog.apiUrl = apiUrl
  velog.appName = appName
  velog.appId = appId
  velog.options = Object.assign({}, velog.options)
  Vue.prototype.$velog = velog
  if (isReport) {
    velog.init(Vue)
  }
  velog.installed = true
}

export default install

if (typeof window !== 'undefined' && window.Vue) {
  window.Vue.use(install)
  if (install.installed) {
    install.installed = false
  }
}
